#include "spi_user.h"
//------------------------------------------------
void spi_init(uint8 spi_no)
{
  if(spi_no > 1) return; //Only SPI and HSPI are valid spi modules.
  uint32 clock_div_flag = 0;
  if(SPI_CLK_USE_DIV)
  {
    clock_div_flag = 0x0001;
  }
  //GPIO Init
   if(spi_no==SPI){
     WRITE_PERI_REG(PERIPHS_IO_MUX, 0x005|(clock_div_flag<<8)); //Set bit 8 if 80MHz sysclock required
     PIN_FUNC_SELECT(PERIPHS_IO_MUX_SD_CLK_U, 1);
     PIN_FUNC_SELECT(PERIPHS_IO_MUX_SD_CMD_U, 1);
     PIN_FUNC_SELECT(PERIPHS_IO_MUX_SD_DATA0_U, 1);
     PIN_FUNC_SELECT(PERIPHS_IO_MUX_SD_DATA1_U, 1);
   }else if(spi_no==HSPI){
     WRITE_PERI_REG(PERIPHS_IO_MUX, 0x105|(clock_div_flag<<9)); //Set bit 9 if 80MHz sysclock required
     PIN_FUNC_SELECT(PERIPHS_IO_MUX_MTDI_U, 2); //GPIO12 is HSPI MISO
     PIN_FUNC_SELECT(PERIPHS_IO_MUX_MTCK_U, 2); //GPIO13 is HSPI MOSI
     PIN_FUNC_SELECT(PERIPHS_IO_MUX_MTMS_U, 2); //GPIO14 is HSPI CLK
     PIN_FUNC_SELECT(PERIPHS_IO_MUX_MTDO_U, 2); //GPIO15 is HSPI CS
   }
   //SPI Clock Init
   if((SPI_CLK_PREDIV==0)|(SPI_CLK_CNTDIV==0))
   {
     WRITE_PERI_REG(SPI_CLOCK(spi_no), SPI_CLK_EQU_SYSCLK);
   }
   else
   {
     WRITE_PERI_REG(SPI_CLOCK(spi_no),
           (((SPI_CLK_PREDIV-1)&SPI_CLKDIV_PRE)<<SPI_CLKDIV_PRE_S)|
           (((SPI_CLK_CNTDIV-1)&SPI_CLKCNT_N)<<SPI_CLKCNT_N_S)|
           (((SPI_CLK_CNTDIV>>1)&SPI_CLKCNT_H)<<SPI_CLKCNT_H_S)|
           ((0&SPI_CLKCNT_L)<<SPI_CLKCNT_L_S));
   }
   //SPI TX Byte order High to Low
   SET_PERI_REG_MASK(SPI_USER(spi_no), SPI_WR_BYTE_ORDER);
   //SPI RX Byte order High to Low
   SET_PERI_REG_MASK(SPI_USER(spi_no), SPI_RD_BYTE_ORDER);
   //Set CS Mode
   SET_PERI_REG_MASK(SPI_USER(spi_no), SPI_CS_SETUP|SPI_CS_HOLD);
   //Disable Flash Mode
   CLEAR_PERI_REG_MASK(SPI_USER(spi_no), SPI_FLASH_MODE);
}
//------------------------------------------------
void spi_send16(uint8 spi_no, uint16 data)
{
  if(spi_no > 1) return;
  //wait for SPI to be ready
  while(spi_busy(spi_no)) ;
  //disable MOSI, MISO, ADDR, COMMAND, DUMMY
  CLEAR_PERI_REG_MASK(SPI_USER(spi_no), SPI_USR_MOSI|SPI_USR_MISO|SPI_USR_COMMAND|SPI_USR_ADDR|SPI_USR_DUMMY);
  //Set bitlengths
  WRITE_PERI_REG(SPI_USER1(spi_no), SPI_USR_ADDR_BITLEN<<SPI_USR_ADDR_BITLEN_S | //Number of bits in Address
                    (15&SPI_USR_MOSI_BITLEN)<<SPI_USR_MOSI_BITLEN_S | //Number of bits to Send
                    SPI_USR_MISO_BITLEN<<SPI_USR_MISO_BITLEN_S |  //Number of bits to receive
                    SPI_USR_DUMMY_CYCLELEN<<SPI_USR_DUMMY_CYCLELEN_S); //Number of Dummy bits to insert
  //Enable MOSI function in SPI module
  SET_PERI_REG_MASK(SPI_USER(spi_no), SPI_USR_MOSI);
  //Copy data to W0
  WRITE_PERI_REG(SPI_W0(spi_no), (uint32)data<<16);
  //Begin SPI Transaction
  SET_PERI_REG_MASK(SPI_CMD(spi_no), SPI_USR);
}
//------------------------------------------------
